@@ -10,19 +10,20 @@ import argparse
 from astropy.time import Time
 from astropy.table import Table
 from astroquery.jplhorizons import Horizons
+
+# Use TkAgg backend by default, but don't change backend if called from a Jupyter notebook with inline plots
+# if 'module://ipykernel.pylab.backend_inline' not in matplotlib.rcParams['backend']:
+#     matplotlib.use('TkAgg')
+
 import matplotlib.pyplot as plt
-
-import matplotlib
-matplotlib.use('TkAgg')
-
 from matplotlib.dates import YearLocator, MonthLocator, DateFormatter
 import numpy as np
 from os.path import join, abspath, dirname
+import pysiaf
 import warnings
 
 from . import ephemeris_old2x as EPH
 
-
 # ignore astropy warning that Date after 2020-12-30 is "dubious"
 warnings.filterwarnings('ignore', category=UserWarning, append=True)
 warnings.filterwarnings('ignore', category=RuntimeWarning, append=True)
@@ -113,8 +114,8 @@ def window_summary_line(fixed, wstart, wend, pa_start, pa_end, ra_start, ra_end,
     if cvz:
         line = " {0:15} {0:11} {0:11} ".format('CVZ')
     else:
-        line = " {:15} {:11} {:11.2f} ".format(Time(wstart, format='mjd', out_subfmt='date').isot,
-                                               Time(wend, format='mjd', out_subfmt='date').isot,wend-wstart)
+        line = " {:15} {:11} {:11.2f} ".format(Time(wstart, format='mjd').datetime.date().strftime("%Y-%m-%d"),
+                                               Time(wend, format='mjd').datetime.date().strftime("%Y-%m-%d"),wend-wstart)
     line += "{:13.5f} {:13.5f} ".format(pa_start*R2D,pa_end*R2D)
     if fixed:
         line += "{:13.5f} {:13.5f} ".format(ra_start*R2D, dec_start*R2D)
@@ -124,16 +125,15 @@ def window_summary_line(fixed, wstart, wend, pa_start, pa_end, ra_start, ra_end,
     return line
 
 def main(args, fixed=True):
-
     table_output=None
     if args.save_table is not None:
         table_output = open(args.save_table, 'w')
 
-    NRCALL_FULL_V2IdlYang = -0.0265
-    NRS_FULL_MSA_V3IdlYang = 137.4874
-    NIS_V3IdlYang = -0.57
-    MIRIM_FULL_V3IdlYang = 5.0152
-    FGS1_FULL_V3IdlYang = -1.2508
+    NRCALL_FULL_V3IdlYang = get_angle('NIRCAM', 'NRCALL_FULL', 'V3IdlYAngle')
+    NRS_FULL_MSA_V3IdlYang = get_angle('NIRSPEC', 'NRS_FULL_MSA', 'V3IdlYAngle')
+    NIS_V3IdlYang = get_angle('NIRISS', 'NIS_CEN', 'V3IdlYAngle')
+    MIRIM_FULL_V3IdlYang = get_angle('MIRI', 'MIRIM_FULL', 'V3IdlYAngle')
+    FGS1_FULL_V3IdlYang = get_angle('FGS', 'FGS1_FULL', 'V3IdlYAngle')
 
     ECL_FLAG = False
 
@@ -160,8 +160,8 @@ def main(args, fixed=True):
     # if len(sys.argv) < 3:
     #   print "proper usage:"
     #   print "find_tgt_info.py ra dec [pa]"
-    #   print "finds full visibility windows over [{}, {}]".format(Time(search_start, format='mjd', out_subfmt='date').isot,
-    #     Time(search_start+span/scale, format='mjd', out_subfmt='date').isot)
+    #   print "finds full visibility windows over [{}, {}]".format(Time(search_start, format='mjd').isot,
+    #     Time(search_start+span/scale, format='mjd').isot)
     #   sys.exit(1)
 
 
@@ -185,21 +185,27 @@ def main(args, fixed=True):
 
     if not args.no_verbose:
         print("", file=table_output)
-        print("       Target", file=table_output)
+        print("", file=table_output)
     if fixed:
         if not args.no_verbose:
             print("                ecliptic", file=table_output)
             print("RA      Dec     latitude", file=table_output)
             print("%7.3f %7.3f %7.3f" % (ra[0]*R2D,dec[0]*R2D,calc_ecliptic_lat(ra[0], dec[0])*R2D), file=table_output)
 
+    if not fixed:
+        if not args.no_verbose:
+            print("TARGET NAME: {}".format(args.name))
+            print("=========================================")
+
+
     if not args.no_verbose:
         print("", file=table_output)
 
     if args.v3pa is not None:
         pa     = float(args.v3pa) * D2R
     if not args.no_verbose:
-        print("Checked interval [{}, {}]".format(Time(search_start, format='mjd', out_subfmt='date').isot,
-            Time(search_start+span, format='mjd', out_subfmt='date').isot), file=table_output)
+        print("Checked interval [{}, {}]".format(Time(search_start, format='mjd').datetime.date().strftime("%Y-%m-%d"),
+            Time(search_start+span, format='mjd').datetime.date().strftime("%Y-%m-%d")), file=table_output)
     if pa == "X":
         iflag_old = A_eph.in_FOR(search_start,ra[0],dec[0])
         if not args.no_verbose:
@@ -345,8 +351,8 @@ def main(args, fixed=True):
 
                 minV3PA = bound_angle(V3PA - max_boresight_roll)
                 maxV3PA = bound_angle(V3PA + max_boresight_roll)
-                minNIRCam_PA = bound_angle(V3PA - max_boresight_roll + NRCALL_FULL_V2IdlYang)
-                maxNIRCam_PA = bound_angle(V3PA + max_boresight_roll + NRCALL_FULL_V2IdlYang)
+                minNIRCam_PA = bound_angle(V3PA - max_boresight_roll + NRCALL_FULL_V3IdlYang)
+                maxNIRCam_PA = bound_angle(V3PA + max_boresight_roll + NRCALL_FULL_V3IdlYang)
                 minNIRSpec_PA = bound_angle(V3PA - max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                 maxNIRSpec_PA = bound_angle(V3PA + max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                 minNIRISS_PA = bound_angle(V3PA - max_boresight_roll + NIS_V3IdlYang)
@@ -374,13 +380,13 @@ def main(args, fixed=True):
                 if fixed:
                     if not args.no_verbose:
                         print(fmt.format(
-                            Time(atime, format='mjd', out_subfmt='date').isot, minV3PA, maxV3PA,
+                            Time(atime, format='mjd').datetime.date().strftime("%Y-%m-%d"), minV3PA, maxV3PA,
                             minNIRCam_PA, maxNIRCam_PA, minNIRSpec_PA, maxNIRSpec_PA, minNIRISS_PA,
                             maxNIRISS_PA, minMIRI_PA, maxMIRI_PA, minFGS_PA, maxFGS_PA), file=table_output)#,sun_ang
                 else:
                     if not args.no_verbose:
                         print(fmt.format(
-                            Time(atime, format='mjd', out_subfmt='date').isot, ra[i]*R2D, dec[i]*R2D, minV3PA, maxV3PA,
+                            Time(atime, format='mjd').datetime.date().strftime("%Y-%m-%d"), ra[i]*R2D, dec[i]*R2D, minV3PA, maxV3PA,
                             minNIRCam_PA, maxNIRCam_PA, minNIRSpec_PA, maxNIRSpec_PA, minNIRISS_PA,
                             maxNIRISS_PA, minMIRI_PA, maxMIRI_PA, minFGS_PA, maxFGS_PA), file=table_output)#,sun_ang
             else:
@@ -512,6 +518,8 @@ def main(args, fixed=True):
 
         if args.save_plot is None:
             plt.show()
+        elif args.save_plot == 'test':
+            plt.close()
         else:
             plt.savefig(args.save_plot)
 
@@ -545,11 +553,11 @@ def get_table(ra, dec, instrument=None, start_date=None, end_date=None, save_tab
     if save_table is not None:
         table_output = open(save_table, 'w')
 
-    NRCALL_FULL_V2IdlYang  = -0.0265
-    NRS_FULL_MSA_V3IdlYang = 137.4874
-    NIS_V3IdlYang          = -0.57
-    MIRIM_FULL_V3IdlYang   = 5.0152
-    FGS1_FULL_V3IdlYang    = -1.2508
+    NRCALL_FULL_V3IdlYang = get_angle('NIRCAM', 'NRCALL_FULL', 'V3IdlYAngle')
+    NRS_FULL_MSA_V3IdlYang = get_angle('NIRSPEC', 'NRS_FULL_MSA', 'V3IdlYAngle')
+    NIS_V3IdlYang = get_angle('NIRISS', 'NIS_CEN', 'V3IdlYAngle')
+    MIRIM_FULL_V3IdlYang = get_angle('MIRI', 'MIRIM_FULL', 'V3IdlYAngle')
+    FGS1_FULL_V3IdlYang = get_angle('FGS', 'FGS1_FULL', 'V3IdlYAngle')
 
     ECL_FLAG = False
 
@@ -575,8 +583,8 @@ def get_table(ra, dec, instrument=None, start_date=None, end_date=None, save_tab
     # if len(sys.argv) < 3:
     #   print "proper usage:"
     #   print "find_tgt_info.py ra dec [pa]"
-    #   print "finds full visibility windows over [{}, {}]".format(Time(search_start, format='mjd', out_subfmt='date').isot,
-    #     Time(search_start+span/scale, format='mjd', out_subfmt='date').isot)
+    #   print "finds full visibility windows over [{}, {}]".format(Time(search_start, format='mjd').isot,
+    #     Time(search_start+span/scale, format='mjd').isot)
     #   sys.exit(1)
 
     pa = 'X'
@@ -608,8 +616,8 @@ def get_table(ra, dec, instrument=None, start_date=None, end_date=None, save_tab
     if v3pa is not None:
         pa     = float(v3pa) * D2R
     if verbose:
-        print("Checked interval [{}, {}]".format(Time(search_start, format='mjd', out_subfmt='date').isot,
-            Time(search_start+span, format='mjd', out_subfmt='date').isot), file=table_output)
+        print("Checked interval [{}, {}]".format(Time(search_start, format='mjd').datetime.date().strftime("%Y-%m-%d"),
+            Time(search_start+span, format='mjd').datetime.date().strftime("%Y-%m-%d")), file=table_output)
     if pa == "X":
         iflag_old = A_eph.in_FOR(search_start,ra[0],dec[0])
         if verbose:
@@ -761,8 +769,8 @@ def get_table(ra, dec, instrument=None, start_date=None, end_date=None, save_tab
 
                 minV3PA = bound_angle(V3PA - max_boresight_roll)
                 maxV3PA = bound_angle(V3PA + max_boresight_roll)
-                minNIRCam_PA = bound_angle(V3PA - max_boresight_roll + NRCALL_FULL_V2IdlYang)
-                maxNIRCam_PA = bound_angle(V3PA + max_boresight_roll + NRCALL_FULL_V2IdlYang)
+                minNIRCam_PA = bound_angle(V3PA - max_boresight_roll + NRCALL_FULL_V3IdlYang)
+                maxNIRCam_PA = bound_angle(V3PA + max_boresight_roll + NRCALL_FULL_V3IdlYang)
                 minNIRSpec_PA = bound_angle(V3PA - max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                 maxNIRSpec_PA = bound_angle(V3PA + max_boresight_roll + NRS_FULL_MSA_V3IdlYang)
                 minNIRISS_PA = bound_angle(V3PA - max_boresight_roll + NIS_V3IdlYang)
@@ -772,7 +780,7 @@ def get_table(ra, dec, instrument=None, start_date=None, end_date=None, save_tab
                 minFGS_PA = bound_angle(V3PA - max_boresight_roll + FGS1_FULL_V3IdlYang)
                 maxFGS_PA = bound_angle(V3PA + max_boresight_roll + FGS1_FULL_V3IdlYang)
 
-                nomNIRCam_PA = bound_angle(V3PA + NRCALL_FULL_V2IdlYang)
+                nomNIRCam_PA = bound_angle(V3PA + NRCALL_FULL_V3IdlYang)
                 nomNIRSpec_PA = bound_angle(V3PA + NRS_FULL_MSA_V3IdlYang)
                 nomNIRISS_PA = bound_angle(V3PA + NIS_V3IdlYang)
                 nomMIRI_PA = bound_angle(V3PA + MIRIM_FULL_V3IdlYang)
@@ -803,14 +811,14 @@ def get_table(ra, dec, instrument=None, start_date=None, end_date=None, save_tab
                 if fixed:
                     if verbose:
                         print(fmt.format(
-                            Time(atime, format='mjd', out_subfmt='date').isot, V3PA, minV3PA, maxV3PA,
+                            Time(atime, format='mjd').datetime.date().strftime("%Y-%m-%d"), V3PA, minV3PA, maxV3PA,
                             nomNIRCam_PA, minNIRCam_PA, maxNIRCam_PA, nomNIRSpec_PA, minNIRSpec_PA, maxNIRSpec_PA,
                             nomNIRISS_PA, minNIRISS_PA, maxNIRISS_PA, nomMIRI_PA, minMIRI_PA, maxMIRI_PA,
                             nomFGS_PA, minFGS_PA, maxFGS_PA), file=table_output)#,sun_ang
                 else:
                     if verbose:
                         print(fmt.format(
-                            Time(atime, format='mjd', out_subfmt='date').isot, ra[i]*R2D, dec[i]*R2D, V3PA, minV3PA, maxV3PA,
+                            Time(atime, format='mjd').datetime.date().strftime("%Y-%m-%d"), ra[i]*R2D, dec[i]*R2D, V3PA, minV3PA, maxV3PA,
                             nomNIRCam_PA, minNIRCam_PA, maxNIRCam_PA, nomNIRSpec_PA, minNIRSpec_PA, maxNIRSpec_PA,
                             nomNIRISS_PA, minNIRISS_PA, maxNIRISS_PA, nomMIRI_PA, minMIRI_PA, maxMIRI_PA,
                             nomFGS_PA, minFGS_PA, maxFGS_PA), file=table_output)#,sun_ang
@@ -891,6 +899,34 @@ def plot_single_instrument(ax, instrument_name, t, min_pa, max_pa):
         ax.set_title(instrument_name)
         ax.fmt_xdata = DateFormatter('%Y-%m-%d')
 
+
+def get_angle(instrument, aperture, angle_name):
+    """Get angle requested by user
+
+    Parameters
+    ----------
+    instrument : JWST instrument of interest
+        type : str
+    
+    aperture : instrument observing aperture
+        type : str
+
+    angle_name : angle of interest
+        type : str
+
+    Returns
+    -------
+    angle : the angle obtained from the SIAF.
+        type : float
+    """
+    
+    siaf = pysiaf.Siaf(instrument)
+    meta = siaf[aperture]
+    angle = getattr(meta, angle_name)
+
+    return angle
+
+
 if __name__ == '__main__':
     try:
         # see if there is a negative dec in sexagesimal coordinates
